import { __assign } from "tslib";
import { CachedInputFileSystem, ResolverFactory } from 'enhanced-resolve';
import fs from 'fs';
import { dirname } from 'path';
import { createCompilerHost, Extension, nodeModuleNameResolver } from 'typescript';
import { loggerPrefix } from './logger';
/**
 * Create the dependencies resolver;
 * @internal
 * @param cwd - working dir
 * @param compilerOptions - compiler options
 * @param options - resolver options
 * @param logger - logger
 */
export function createResolver(cwd, compilerOptions, options, logger) {
    /**
     * Prepare compiler host.
     */
    var compilerHost = createCompilerHost(compilerOptions);
    logger.debug('create resolver with options:', options);
    /**
     * Prepare base resolver
     */
    var resolver = ResolverFactory.createResolver(__assign({ cacheWithContext: false, fileSystem: new CachedInputFileSystem(fs, 4000), useSyncFileSystemCalls: true }, options));
    var externalMatch = function (target) { return function (val) {
        if (typeof val === 'string') {
            return val === target;
        }
        else {
            return val.test(target);
        }
    }; };
    /**
     * Is checking alias is a absolute url.
     * @param target - target dependency
     */
    var checkAliasForUrl = function (target) {
        var _a;
        var resolution;
        if (Array.isArray(options.alias)) {
            var alias = options.alias.find(function (i) { return i.name === target; });
            if (alias && typeof alias.alias === 'string') {
                resolution = alias.alias;
            }
        }
        else {
            var alias = (_a = options.alias) === null || _a === void 0 ? void 0 : _a[target];
            if (alias && typeof alias === 'string') {
                resolution = alias;
            }
        }
        if (resolution && (resolution.startsWith('http://') || resolution.startsWith('https://'))) {
            return resolution;
        }
    };
    return function (context, target) {
        var _a;
        var prefix = "import of: '" + target + "'\n from: '" + context + "\n";
        // is external
        if (options.externals && options.externals.find(externalMatch(target))) {
            logger.debug(prefix + " resolved as external");
            return { context: context, target: target, resolution: target, isExternal: true };
        }
        // is a absolute url
        var urlAlias = checkAliasForUrl(target);
        if (urlAlias) {
            logger.debug(prefix + " resolved by alias to: " + urlAlias);
            return { context: context, target: target, resolution: urlAlias, isUrl: true };
        }
        // is from ts/tsx module
        if (context.endsWith('.ts') || context.endsWith('.tsx')) {
            // try resolving by the typescript
            var tsResolution = nodeModuleNameResolver(target, context, compilerOptions, compilerHost);
            if ((_a = tsResolution === null || tsResolution === void 0 ? void 0 : tsResolution.resolvedModule) === null || _a === void 0 ? void 0 : _a.resolvedFileName) {
                if (tsResolution.resolvedModule.extension !== Extension.Dts) {
                    logger.debug(prefix + " resolved by typescript to: " + tsResolution.resolvedModule.resolvedFileName);
                    return {
                        isTypescript: true,
                        context: context, target: target,
                        resolution: tsResolution.resolvedModule.resolvedFileName
                    };
                }
            }
        }
        try {
            // try resolving by the node
            var nodeResolution = resolver.resolveSync({}, dirname(context), target);
            if (nodeResolution) {
                logger.debug(prefix + " resolved by enhanced-resolver to: " + nodeResolution);
                return {
                    isNode: true,
                    context: context,
                    target: target,
                    resolution: nodeResolution
                };
            }
            // eslint-disable-next-line no-empty
        }
        catch (e) {
            logger.debug(loggerPrefix, 'enhanced-resolver error:', e);
        }
        logger.error(prefix + " not resolved !!!");
        throw new Error(loggerPrefix + " resolution not found for " + target + " imported by " + context);
    };
}
//# sourceMappingURL=create-resolver.js.map